@@ -9,7 +9,7 @@ try:
 except ImportError:
     pytils_enabled = False
 
-from django.shortcuts import get_object_or_404, get_list_or_404
+from django.shortcuts import get_object_or_404, get_list_or_404, redirect
 from django.http import HttpResponseRedirect, HttpResponse,\
                         HttpResponseNotFound, Http404
 from django.contrib.auth.models import User
@@ -20,48 +20,76 @@ from django.db import connection
 from django.utils.translation import ugettext_lazy as _
 
 from common.decorators import render_to, ajax
+from common.orm import load_related
+from common.pagination import paginate
 
 from pybb.markups import mypostmarkup
-from pybb.util import quote_text, paginate,\
-                        set_language, urlize
+from pybb.util import quote_text, set_language, urlize
 from pybb.models import Category, Forum, Topic, Post, Profile, \
                         Attachment, MARKUP_CHOICES
 from pybb.forms import  AddPostForm, EditPostForm, EditHeadPostForm, \
                         EditProfileForm, UserSearchForm
-from pybb.orm import load_related
 from pybb.read_tracking import update_read_tracking
+from pybb.templatetags.pybb_tags import pybb_editable_by, pybb_moderated_by
 
 
-def index_ctx(request):
-    cats = {}
-    for forum in Forum.objects.all().select_related():
-        cat = cats.setdefault(forum.category.pk,
-                              {'cat': forum.category, 'forums': []})
-        cat['forums'].append(forum)
-    cats = sorted(cats.values(), key=lambda x: x['cat'].position)
 
+def load_last_post(objects):
+    """
+    Get list of topics/forums and find the recent post in
+    each topic/forum. Also extract author of the post.
+    """
+
+    pk_list = [x.last_post_id for x in objects]
+    qs = Post.objects.filter(pk__in=pk_list).select_related('user')
+    posts = dict((x.pk, x) for x in qs)
+    for obj in objects:
+        obj.last_post = posts.get(obj.last_post_id)
+
+
+@render_to('pybb/index.html')
+def index(request):
+    """
+    Display list of categories and forums in each category.
+    """
+
+    cats = list(Category.objects.all())
+    cat_map = dict((x.pk, x) for x in cats)
+    for cat in cats:
+        cat.cached_forums = []
+    forums = list(Forum.objects.all())
+    load_last_post(forums)
+    for forum in forums:
+        cat_map[forum.category_id].cached_forums.append(forum)
     return {'cats': cats,
             }
 
 
-def show_category_ctx(request, category_id):
+@render_to('pybb/category_details.html')
+def category_details(request, category_id):
     category = get_object_or_404(Category, pk=category_id)
+    forums = category.forums.all()
+    load_last_post(forums)
+    category.cached_forums = forums
 
     return {'category': category,
             }
 
 
-def show_forum_ctx(request, forum_id):
+@render_to('pybb/forum_details.html')
+def forum_details(request, forum_id):
     forum = get_object_or_404(Forum, pk=forum_id)
     topics = forum.topics.order_by('-sticky', '-updated').select_related()
-    page, paginator = paginate(topics, request, settings.PYBB_FORUM_PAGE_SIZE)
+    page = paginate(topics, request, settings.PYBB_FORUM_PAGE_SIZE)
+    load_last_post(page.object_list)
 
     return {'forum': forum,
             'page': page,
             }
 
 
-def show_topic_ctx(request, topic_id):
+@render_to('pybb/topic_details.html')
+def topic_details(request, topic_id):
     try:
         topic = Topic.objects.select_related().get(pk=topic_id)
     except Topic.DoesNotExist:
@@ -72,12 +100,11 @@ def show_topic_ctx(request, topic_id):
 
     if request.user.is_authenticated():
         update_read_tracking(topic, request.user)
-
+	
     if settings.PYBB_FREEZE_FIRST_POST:
-        first_post = topic.posts.order_by('created')[0]
+        first_post = topic.head
     else:
         first_post = None
-    last_post = topic.posts.order_by('-created')[0]
 
     form = AddPostForm(topic=topic)
 
@@ -86,21 +113,21 @@ def show_topic_ctx(request, topic_id):
     subscribed = (request.user.is_authenticated() and
                   request.user in topic.subscribers.all())
 
-    posts = topic.posts.all().select_related()
-    page, paginator = paginate(posts, request, settings.PYBB_TOPIC_PAGE_SIZE,
-                               total_count=topic.post_count)
+    posts = topic.posts.all()
 
-    profiles = Profile.objects.filter(user__pk__in=
-        set(x.user.id for x in page.object_list))
-    profiles = dict((x.user_id, x) for x in profiles)
+    page = paginate(posts, request, settings.PYBB_TOPIC_PAGE_SIZE)
+
+    users = User.objects.filter(pk__in=
+        set(x.user_id for x in page.object_list)).select_related("pybb_profile")
+    users = dict((user.pk, user) for user in users)
 
     for post in page.object_list:
-        post.user.pybb_profile = profiles[post.user.id]
+        post.user = users.get(post.user_id)
 
     load_related(page.object_list, Attachment.objects.all(), 'post')
 
     return {'topic': topic,
-            'last_post': last_post,
+            'last_post': topic.last_post,
             'first_post': first_post,
             'form': form,
             'moderator': moderator,
@@ -111,7 +138,8 @@ def show_topic_ctx(request, topic_id):
 
 
 @login_required
-def add_post_ctx(request, forum_id, topic_id):
+@render_to('pybb/post_add.html')
+def post_add(request, forum_id, topic_id):
     forum = None
     topic = None
 
@@ -143,7 +171,7 @@ def add_post_ctx(request, forum_id, topic_id):
 
     if form.is_valid():
         post = form.save();
-        return HttpResponseRedirect(post.get_absolute_url())
+        return redirect(post)
 
     return {'form': form,
             'topic': topic,
@@ -151,7 +179,8 @@ def add_post_ctx(request, forum_id, topic_id):
             }
 
 
-def user_ctx(request, username):
+@render_to('pybb/user_details.html')
+def user_details(request, username):
     user = get_object_or_404(User, username=username)
     topic_count = Topic.objects.filter(user=user).count()
 
@@ -160,26 +189,28 @@ def user_ctx(request, username):
             }
 
 
-def user_topics_ctx(request, username):
+@render_to('pybb/user_details_topics.html')
+def user_details_topics(request, username):
     user = get_object_or_404(User, username=username)
     topics = Topic.objects.filter(user=user).order_by('-created')
-    page, paginator = paginate(topics, request, settings.PYBB_TOPIC_PAGE_SIZE)
+    page = paginate(topics, request, settings.PYBB_TOPIC_PAGE_SIZE)
 
     return {'profile': user,
             'page': page,
             }
 
 
-def show_post(request, post_id):
+def post_details(request, post_id):
     post = get_object_or_404(Post, pk=post_id)
     count = post.topic.posts.filter(created__lt=post.created).count() + 1
     page = math.ceil(count / float(settings.PYBB_TOPIC_PAGE_SIZE))
-    url = '%s?page=%d#post-%d' % (reverse('pybb_topic', args=[post.topic.id]), page, post.id)
-    return HttpResponseRedirect(url)
+    url = '%s?page=%d#post-%d' % (reverse('pybb_topic_details', args=[post.topic.id]), page, post.id)
+    return redirect(url)
 
 
 @login_required
-def edit_profile_ctx(request):
+@render_to('pybb/profile_edit.html')
+def profile_edit(request):
 
     form_kwargs = dict(instance=request.user.pybb_profile)
     if request.method == 'POST':
@@ -190,7 +221,7 @@ def edit_profile_ctx(request):
     if form.is_valid():
         profile = form.save()
         set_language(request, profile.language)
-        return HttpResponseRedirect(reverse('pybb_edit_profile'))
+        return redirect('pybb_profile_edit')
 
     return {'form': form,
             'profile': request.user.pybb_profile,
@@ -198,14 +229,14 @@ def edit_profile_ctx(request):
 
 
 @login_required
-def edit_post_ctx(request, post_id):
-    from pybb.templatetags.pybb_tags import pybb_editable_by
+@render_to('pybb/post_edit.html')
+def post_edit(request, post_id):
 
     post = get_object_or_404(Post, pk=post_id)
 
     if not pybb_editable_by(post, request.user) \
     or request.user.pybb_profile.is_banned():
-        return HttpResponseRedirect(post.get_absolute_url())
+        return redirect(post)
 
     head_post_id = post.topic.posts.order_by('created')[0].id
     form_kwargs = dict(instance=post, initial={'title': post.topic.name})
@@ -216,13 +247,13 @@ def edit_post_ctx(request, post_id):
         form_class = EditPostForm
 
     if request.method == 'POST':
-        form = form_class(request.POST, request.FILES, **form_kwargs) 
+        form = form_class(request.POST, request.FILES, **form_kwargs)
     else:
         form = form_class(**form_kwargs)
 
     if form.is_valid():
         post = form.save()
-        return HttpResponseRedirect(post.get_absolute_url())
+        return redirect(post)
 
     return {'form': form,
             'post': post,
@@ -230,31 +261,29 @@ def edit_post_ctx(request, post_id):
 
 
 @login_required
-def stick_topic(request, topic_id):
-    from pybb.templatetags.pybb_tags import pybb_moderated_by
+def topic_stick(request, topic_id):
 
     topic = get_object_or_404(Topic, pk=topic_id)
     if pybb_moderated_by(topic, request.user):
         if not topic.sticky:
             topic.sticky = True
             topic.save()
-    return HttpResponseRedirect(topic.get_absolute_url())
+    return redirect(topic)
 
 
 @login_required
-def unstick_topic(request, topic_id):
-    from pybb.templatetags.pybb_tags import pybb_moderated_by
-
+def topic_unstick(request, topic_id):
     topic = get_object_or_404(Topic, pk=topic_id)
     if pybb_moderated_by(topic, request.user):
         if topic.sticky:
             topic.sticky = False
             topic.save()
-    return HttpResponseRedirect(topic.get_absolute_url())
+    return redirect(topic)
 
 
 @login_required
-def delete_post_ctx(request, post_id):
+@render_to('pybb/post_delete.html')
+def post_delete(request, post_id):
     post = get_object_or_404(Post, pk=post_id)
     last_post = post.topic.posts.order_by('-created')[0]
 
@@ -265,7 +294,7 @@ def delete_post_ctx(request, post_id):
         allowed = True
 
     if not allowed:
-        return HttpResponseRedirect(post.get_absolute_url())
+        return redirect(post)
 
     if 'POST' == request.method:
         topic = post.topic
@@ -275,44 +304,39 @@ def delete_post_ctx(request, post_id):
         try:
             Topic.objects.get(pk=topic.id)
         except Topic.DoesNotExist:
-            return HttpResponseRedirect(forum.get_absolute_url())
+            return redirect(forum)
         else:
-            return HttpResponseRedirect(topic.get_absolute_url())
+            return redirect(topic)
     else:
         return {'post': post,
                 }
 
 
 @login_required
-def close_topic(request, topic_id):
-    from pybb.templatetags.pybb_tags import pybb_moderated_by
-
+def topic_close(request, topic_id):
     topic = get_object_or_404(Topic, pk=topic_id)
     if pybb_moderated_by(topic, request.user):
         if not topic.closed:
             topic.closed = True
             topic.save()
-    return HttpResponseRedirect(topic.get_absolute_url())
+    return redirect(topic)
 
 
 
 @login_required
-def open_topic(request, topic_id):
-    from pybb.templatetags.pybb_tags import pybb_moderated_by
-
+def topic_open(request, topic_id):
     topic = get_object_or_404(Topic, pk=topic_id)
     if pybb_moderated_by(topic, request.user):
         if topic.closed:
             topic.closed = False
             topic.save()
-    return HttpResponseRedirect(topic.get_absolute_url())
+    return redirect(topic)
 
 
 
 @login_required
-def merge_topics_ctx(request):
-    from pybb.templatetags.pybb_tags import pybb_moderated_by
-
+@render_to('pybb/topic_merge.html')
+def topic_merge(request):
     topics_ids = request.GET.getlist('topic')
     topics = get_list_or_404(Topic, pk__in=topics_ids)
 
@@ -346,7 +370,7 @@ def merge_topics_ctx(request):
                 topic.delete()
                 forum.update_post_count()
 
-        return HttpResponseRedirect(main_topic.get_absolute_url())
+        return redirect(main_topic)
 
     return {'posts': posts,
             'topics': topics,
@@ -354,12 +378,13 @@ def merge_topics_ctx(request):
             }
 
 
-def users_ctx(request):
+@render_to('pybb/user_list.html')
+def user_list(request):
     users = User.objects.order_by('username')
     form = UserSearchForm(request.GET)
     users = form.filter(users)
 
-    page, paginator = paginate(users, request, settings.PYBB_USERS_PAGE_SIZE)
+    page = paginate(users, request, settings.PYBB_USERS_PAGE_SIZE)
 
     return {'page': page,
             'form': form,
@@ -367,24 +392,24 @@ def users_ctx(request):
 
 
 @login_required
-def delete_subscription(request, topic_id):
+def subscription_delete(request, topic_id):
     topic = get_object_or_404(Topic, pk=topic_id)
     topic.subscribers.remove(request.user)
     if 'from_topic' in request.GET:
-        return HttpResponseRedirect(reverse('pybb_topic', args=[topic.id]))
+        return redirect(topic)
     else:
-        return HttpResponseRedirect(reverse('pybb_edit_profile'))
+        return redirect('pybb_edit_profile')
 
 
 @login_required
-def add_subscription(request, topic_id):
+def subscription_add(request, topic_id):
     topic = get_object_or_404(Topic, pk=topic_id)
     topic.subscribers.add(request.user)
-    return HttpResponseRedirect(reverse('pybb_topic', args=[topic.id]))
+    return redirect('pybb_topic_details', topic.id)
 
 
 @login_required
-def show_attachment(request, hash):
+def attachment_details(request, hash):
     attachment = get_object_or_404(Attachment, hash=hash)
     file_obj = file(attachment.get_absolute_path())
     # without it mod_python chokes with error that content_type must be string
@@ -395,9 +420,8 @@ def show_attachment(request, hash):
 @ajax
 def post_ajax_preview(request):
     content = request.POST.get('content')
-    markup = request.POST.get('markup')
+    markup = request.user.pybb_profile.markup
 
-    print 'markup', markup
     if not markup in dict(MARKUP_CHOICES).keys():
         return {'error': 'Invalid markup'}
 
@@ -414,17 +438,3 @@ def post_ajax_preview(request):
 
     return {'content': html,
             }
-
-
-users = render_to('pybb/users.html')(users_ctx)
-merge_topics = render_to('pybb/merge_topics.html')(merge_topics_ctx)
-delete_post = render_to('pybb/delete_post.html')(delete_post_ctx)
-edit_post = render_to('pybb/edit_post.html')(edit_post_ctx)
-edit_profile = render_to('pybb/edit_profile.html')(edit_profile_ctx)
-user_topics = render_to('pybb/user_topics.html')(user_topics_ctx)
-user = render_to('pybb/user.html')(user_ctx)
-add_post = render_to('pybb/add_post.html')(add_post_ctx)
-show_topic = render_to('pybb/topic.html')(show_topic_ctx)
-show_forum = render_to('pybb/forum.html')(show_forum_ctx)
-index = render_to('pybb/index.html')(index_ctx)
-show_category = render_to('pybb/category.html')(show_category_ctx)
